import re
from typing import AnyStr
from re import Pattern

from django.conf import settings

DEFAULT_ENTITLEMENT_RE = r"^urn:mace:surf\.nl:sram:group:(?P<organisation>[^:\s]+)(?P<collaboration>:[^:\s]+)?(?P<group>:[\S]+)?$"
ENTITLEMENT_RE = re.compile(
    getattr(settings, "SRAM_ENTITLEMENT_REGEX", DEFAULT_ENTITLEMENT_RE)
)


def get_collaborations(
    entitlements: list[str], entitlement_regex_pattern: Pattern[AnyStr] = ENTITLEMENT_RE
) -> list[str]:
    """Convert entitlements into a list of unique collaboration names.
    :param entitlements: List of entitlement URNs from SRAM.
    :param entitlement_regex_pattern: Optional regex pattern to extract the collaborations.
                                      Defaults to SRAM_ENTITLEMENT_REGEX from settings if it is set, otherwise:
                                      `'urn:mace:surf.nl:sram:group:<organisation>:<collaboration>:<group>'`
                                      `r"^urn:mace:surf\\.nl:sram:group:(?P<organisation>[^:\\s]+)(?P<collaboration>:[^:\\s]+)?(?P<group>:[\\S]+)?$"`
    :return: A deduplicated list of collaboration names, stripped of their leading colon.
    """
    collaborations = []
    for entitlement in entitlements:
        if match := entitlement_regex_pattern.match(entitlement):
            if collaboration := match.group("collaboration"):
                collaborations.append(collaboration[1:])
    return list(set(collaborations))


def get_groups_in_collaboration(
    collaboration: str,
    entitlements: list[str],
    entitlement_regex_pattern: Pattern[AnyStr] = ENTITLEMENT_RE,
) -> list[str]:
    """Get all groups for a given collaboration
    :param entitlement_regex_pattern: Optional regex pattern to extract the collaborations. Defaults to ENTITLEMENT_RE
    :param collaboration: Collaboration name
    :param entitlements: List of entitlement URNs from SRAM.
    """
    groups = []
    for entitlement in entitlements:
        if re_match := entitlement_regex_pattern.match(entitlement):
            if collab := re_match.group("collaboration"):
                if collab.endswith(collaboration):
                    if group := re_match.group("group"):
                        groups.append(group[1:])
    return groups



from rest_framework_simplejwt import models


class TokenUser(models.TokenUser):
    """
    Placeholder Class

    Implementation see: https://github.com/jazzband/djangorestframework-simplejwt/blob/master/rest_framework_simplejwt/models.py
    """

    def is_member_of_group_in_co(self, org: str, co: str, group: str) -> bool:
        """returns the user is member of group in given collaborative organisation"""
        match = f"urn:mace:surf.nl:sram:group:{org}:{co}:{group}"
        entitlements = self.token.get("eduperson_entitlement")
        return match in entitlements



from django.http import HttpRequest
from rest_framework import mixins, status, viewsets
from rest_framework.permissions import AllowAny
from rest_framework.response import Response

from django_sram.user import TokenUser


class UserInfo(mixins.ListModelMixin, viewsets.ViewSet):
    permission_classes = [AllowAny]  # endpoint for checking if you are logged in

    def list(self, request: HttpRequest, *args, **kwargs):
        user: TokenUser = request.user
        try:
            data = {
                "uid": user.id,
                "name": getattr(user, "name"),
                "preferred_username": getattr(user, "preferred_username"),
                "given_name": getattr(user, "given_name"),
                "family_name": getattr(user, "family_name"),
                "email": getattr(user, "email"),
                "email_verified": getattr(user, "email_verified"),
                "eduperson_entitlement": getattr(user, "eduperson_entitlement"),
                "eduperson_unique_id": getattr(user, "eduperson_unique_id"),
                "token_type": getattr(user, "typ"),
            }

            res_status = status.HTTP_200_OK
        except AttributeError:
            data = {
                "detail": "token auth failed",
                "headers": request.headers,
            }
            res_status = status.HTTP_401_UNAUTHORIZED

        return Response(data, status=res_status)



from django.urls import include, path
from rest_framework.routers import DefaultRouter

from django_sram.views.userinfo_viewset import UserInfo

router = DefaultRouter()
router.register(r"userinfo", UserInfo, "userinfo")

urlpatterns = [
    path("", include(router.urls)),
]



from rest_framework_simplejwt.authentication import JWTStatelessUserAuthentication


class OAUTH2ProxyStatelessAuthentication(JWTStatelessUserAuthentication):
    """OAuth2 Proxy uses the HTT_X_FORWARDED_ACCESS_TOKEN header directly"""

    def get_raw_token(self, header: bytes) -> bytes:
        return header
